{Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved.}

{TERATERM.EXE, file transfer routines}
unit FileSys;
{$r FileSys.res}

interface
{$I teraterm.inc}

{$IFDEF Delphi}
uses Messages, WinTypes, WinProcs, OWindows, ODialogs, Strings, TTWinMan,
     TTCommon, TTTypes, TTFTypes, CommLib;
{$ELSE}
uses WinTypes, WinProcs, WObjects, Strings, TTWinMan,
     TTCommon, TTTypes, TTFTypes, Win31, CommLib;
{$ENDIF}

const

  Id_FileTransDlg = 2100;
  Id_TransFname = 2101;
  Id_TransBytes = 2102;
  Id_TransPauseStart = 2103;
  Id_TransHelp = 2199;

  IdTTIcon = 100;

{TTFILE.DLL routines}
type
  TGetSetupFname = function(HWin: HWnd; Save: bool; ts: PTTSet): BOOL;
  TGetTransFname = function(HWin: HWnd; FuncId: word; Option: Plongint; fv: PFileVar; CurDir: PChar): BOOL;
  TGetMultiFname = function(HWin: HWnd; fv: PFileVar; CurDir: PChar; FuncId: word; Option: PWord): BOOL;
  TGetGetFname = function(HWin: HWnd; fv: PFileVar): BOOL;
  TSetFileVar = procedure(fv: PFileVar);
  TGetXFname = function(HWin: HWnd; Receive: bool; Option: Plongint; fv: PFileVar; CurDir: PChar): BOOL;
  TProtoInit = procedure(Proto: integer; fv: PFileVar; pv: pointer; cv: PComVar; ts: PTTSet);
  TProtoParse = function(Proto: integer; fv: PFileVar; pv: pointer; cv: PComVar): BOOL;
  TProtoTimeOutProc = procedure(Proto: integer; fv: PFileVar; pv: pointer; cv: PComVar);
  TProtoCancel = function(Proto: integer; fv: PFileVar; pv: pointer; cv: PComVar): BOOL;
  TVTPrint = procedure(vpv: PVTPrnVar);

{text file transfer dialog box}
  PFileTransDlg = ^TFileTransDlg;
  TFileTransDlg = object(TDlgWindow)
    LogFlag, FirstAct, Pause: boolean;
    fv: PFileVar;
    constructor Init(Log: boolean);
    destructor Done; virtual;
    function GetClassName: PChar; virtual;
    procedure GetWindowClass(var AWndClass: TWndClass); virtual;
    procedure SetupWindow; virtual;
    procedure RefreshNum;
    procedure ChangeButton;
    procedure WMCommand(var Msg: TMessage);
      virtual wm_First + wm_Command;
    procedure WMWindowPosChanged(var Msg: TMessage);
      virtual wm_First + wm_WindowPosChanged;
  end;

  PProtoDlg = ^TProtoDlg;
  TProtoDlg = object(TDialog)
    Proto: integer;
    LogFlag: BOOL;
    fv: PFileVar;
    pv: pointer;
    vsize: integer;

    constructor Init(pfv: PFileVar; ProtoId, Mode: integer; Opt1, Opt2: WORD);
    destructor Done; virtual;
    procedure Cancel(var Msg: TMessage);
      virtual id_First + id_Cancel;
    procedure SetupWindow; virtual;
    procedure TimeOutProc;
    function Parse: integer;
  end;

var
  HTTFILE: THandle; 
  TTFILECount: integer;
  LogVar, SendVar, FileVar: PFileVar;
  LogLast: byte;
  FileRetrySend, FileRetryEcho, FileCRSend: boolean;
  FileByte: byte;
  FLogDlg, SendDlg: PFileTransDlg;
  ProtoDlg: PProtoDlg;

  {TTFILE.DLL}
  GetSetupFname: TGetSetupFname;
  GetTransFname: TGetTransFname;
  GetMultiFname: TGetMultiFname;
  GetGetFname: TGetGetFname;
  SetFileVar: TSetFileVar;
  GetXFname: TGetXFname;

  ProtoInit: TProtoInit;
  ProtoParse: TProtoParse;
  ProtoTimeOutProc: TProtoTimeOutProc;
  ProtoCancel: TProtoCancel;

  VTPrint: TVTPrint;

function LoadTTFILE: bool;
function FreeTTFILE: bool;
function NewFileVar(var FV: PFileVar): bool;
procedure FreeFileVar(var FV: PFileVar);

{Log & DDE buffer}
procedure LogStart;
procedure LogEnd;
procedure Log1Byte(b: byte);
procedure LogToFile;
function CreateLogBuf: boolean;
procedure FreeLogBuf;
function CreateBinBuf: boolean;
procedure FreeBinBuf;

{File send}
procedure FileSendStart;
procedure FileSendEnd;
procedure FileSend;

{File transfer protcols}
function ProtoStart: bool;
procedure ProtoEnd;
function ProtoDlgParse: integer;
procedure ProtoDlgTimeOut;
procedure KermitStart(mode: integer);
procedure XMODEMStart(mode: integer);
procedure ZMODEMStart(mode: integer);
procedure BPStart(mode: integer);
procedure QVStart(mode: integer);

var
  FSend, ProtoFlag, FileLog, BinLog, DDELog: boolean;

implementation

uses TTDDE;

const
  IdGetSetupFname  = 1;
  IdGetTransFname  = 2;
  IdGetMultiFname  = 3;
  IdGetGetFname    = 4;
  IdSetFileVar     = 5;
  IdGetXFname      = 6;

  IdProtoInit      = 7;
  IdProtoParse     = 8;
  IdProtoTimeOutProc = 9;
  IdProtoCancel    = 10;

  IdVTPrint        = 20;

function LoadTTFILE: bool;
var
  Err: boolean;
begin
  if HTTFILE >= 32 then
  begin
    inc(TTFILECount);
    LoadTTFILE := true;
    exit;
  end
  else begin
    LoadTTFILE := false;
    HTTFILE := 0;
    TTFILECount := 0;
  end;

  HTTFILE := LoadLibrary('TTFILE.DLL');
  if HTTFILE < 32 then exit;

  Err := false;
  @GetSetupFname := GetProcAddress(HTTFILE, PChar(IdGetSetupFname));
  if @GetSetupFname=nil then Err := true;

  @GetTransFname := GetProcAddress(HTTFILE, PChar(IdGetTransFname));
  if @GetTransFname=nil then Err := true;

  @GetMultiFname := GetProcAddress(HTTFILE, PChar(IdGetMultiFname));
  if @GetMultiFname=nil then Err := true;

  @GetGetFname := GetProcAddress(HTTFILE, PChar(IdGetGetFname));
  if @GetGetFname=nil then Err := true;

  @SetFileVar := GetProcAddress(HTTFILE, PChar(IdSetFileVar));
  if @SetFileVar=nil then Err := true;

  @GetXFname := GetProcAddress(HTTFILE, PChar(IdGetXFname));
  if @GetXFname=nil then Err := true;

  @ProtoInit := GetProcAddress(HTTFILE, PChar(IdProtoInit));
  if @ProtoInit=nil then Err := true;

  @ProtoParse := GetProcAddress(HTTFILE, PChar(IdProtoParse));
  if @ProtoParse=nil then Err := true;

  @ProtoTimeOutProc := GetProcAddress(HTTFILE, PChar(IdProtoTimeOutProc));
  if @ProtoTimeOutProc=nil then Err := true;

  @ProtoCancel := GetProcAddress(HTTFILE, PChar(IdProtoCancel));
  if @ProtoCancel=nil then Err := true;

  @VTPrint := GetProcAddress(HTTFILE, PChar(IdVTPrint));
  if @VTPrint=nil then Err := true;

  if Err then
  begin
    FreeLibrary(HTTFILE);
    HTTFILE := 0;
  end
  else begin
    LoadTTFILE := true;
    TTFILECount := 1;
  end;
end;

function FreeTTFILE: bool;
begin
  FreeTTFILE := false;
  if TTFILECount=0 then exit;
  FreeTTFILE := true;
  dec(TTFILECount);
  if (TTFILECount=0) and (HTTFILE>=32) then
  begin
    FreeLibrary(HTTFile);
    HTTFILE := 0;
  end;
end;

function NewFileVar(var FV: PFileVar): bool;
begin
  NewFileVar := false;

  if FV=nil then
  begin
    New(FV);
    if FV<>nil then
    begin
      with FV^ do
      begin        
        FillChar(FV^, SizeOf(TFileVar), #0);        
        StrCopy(FullName,ts.FileDir);
        DirLen := StrLen(FullName);
        if FullName[DirLen-1]<>'\' then
        begin
          StrCat(FullName,'\');
          inc(DirLen);
        end;
        FileOpen := false;
      end;
      FV^.HMainWin := HMainWin;
    end;
  end;

  if FV<>nil then
    NewFileVar := true;
end;

procedure FreeFileVar(var FV: PFileVar);
begin
  if FV<>nil then
  begin
    with FV^ do begin
      if FileOpen then _lclose(FileHandle);
      if FnStrMemHandle>0 then
      begin
        GlobalUnlock(FnStrMemHandle);
        GlobalFree(FnStrMemHandle);
      end;
    end;
    Dispose(FV);
    FV := nil;
  end;
end;

procedure LogStart;
var
  Option: longint;
begin
  if (FileLog) or (BinLog) then exit;

  if not LoadTTFILE then exit;
  if not NewFileVar(LogVar) then
  begin
    FreeTTFILE;
    exit;
  end;
  LogVar^.OpId := OpLog;

  if (StrLen(@LogVar^.FullName[LogVar^.DirLen])=0) then
  begin
    Option := MakeLong(ts.TransBin,ts.Append);
    if not GetTransFname(HMainWin, GTF_Log, @Option, LogVar, ts.FileDir) then
    begin
      LogEnd;
      exit;
    end;
    ts.TransBin := LoWord(Option);
    ts.Append := HiWord(Option);
  end
  else
    SetFileVar(LogVar);

  if ts.TransBin > 0 then
  begin
    if not CreateBinBuf then
    begin
      LogEnd;
      exit;
    end;
    BinLog := true;
    FileLog := false;
  end
  else begin
    if not CreateLogBuf then
    begin
      LogEnd;
      exit;
    end;
    BinLog := false;
    FileLog := true;
  end;
  cv.LStart := cv.LogPtr;
  cv.LCount := 0;

  with LogVar^ do
  begin
    HelpId := HlpFileLog;
    if ts.Append>0 then
    begin
      FileHandle := _lopen(FullName,of_Write);
      if FileHandle>0 then
        _llseek(FileHandle,0,2)
      else
        FileHandle := _lcreat(FullName,0);
    end
    else
      FileHandle := _lcreat(FullName,0);
    FileOpen := FileHandle>0;
    if not FileOpen then
    begin
      LogEnd;
      exit;
    end;
    ByteCount := 0;
  end;
  FLogDlg := PFileTransDlg(Application^.MakeWindow(New(PFileTransDlg,
                 Init(true)) ));
  if FLogDlg=nil then LogEnd;
end;

procedure LogEnd;
begin
  if FLogDlg<>nil then
  begin
    FLogDlg^.CloseWindow;
    FLogDlg := nil;
  end;

  FreeTTFILE;
  FreeFileVar(LogVar);

  FileLog := false;
  FreeLogBuf;
  BinLog := false;
  FreeBinBuf;
end;

procedure LogPut1(b: byte);
begin
  with cv do begin
    PChar(LogBuf)[LogPtr] := char(b);
    inc(LogPtr);
    if LogPtr>=InBuffSize then LogPtr := LogPtr-InBuffSize;
    if LCount>=InBuffSize then
    begin
      LCount := InBuffSize;
      LStart := LogPtr;
    end
    else inc(LCount);
    if DCount>=InBuffSize then
    begin
      DCount := InBuffSize;
      DStart := LogPtr;
    end
    else inc(DCount);
  end;
end;

procedure Log1Byte(b: byte);
begin
  if b=$0d then
  begin
    LogLast := b;
    exit;
  end;
  if (b=$0a) and (LogLast=$0d) then
    LogPut1($0d);
  LogPut1(b);
end;

procedure LogToFile;

  function Get1(Buf: pointer; var Start, Count: integer; var b: byte): boolean;
  begin
    if Count>0 then
      Get1 := true
    else begin
      Get1 := false;
      exit;
    end;
    b := byte( PChar(Buf)[Start] );
    inc(Start);
    if Start>=InBuffSize then Start := Start-InBuffSize;
    dec(Count);
  end;

var
  Buf: pointer;
  Start,Count: integer;
  b: byte;
begin
with cv do begin
  if not LogVar^.FileOpen then exit;
  if FileLog then
  begin
    Buf := LogBuf;
    Start := LStart;
    Count := LCount;
  end
  else if BinLog then
  begin
    Buf := BinBuf;
    Start := BStart;
    Count := BCount;
  end
  else
    exit;

  if Buf=nil then exit;
  if Count=0 then exit;

  while Get1(Buf,Start,Count,b) do
  begin
    if not FilePause then
    begin
      _lwrite(LogVar^.FileHandle,@b,1);
      inc(LogVar^.ByteCount);
    end;
  end;

  if FileLog then
  begin
    LStart := Start;
    LCount := Count;
  end
  else begin
    BStart := Start;
    BCount := Count;
  end;
  if FilePause then exit;
  if FLogDlg<>nil then FLogDlg^.RefreshNum;
end;
end;

function CreateLogBuf: boolean;
begin
  with cv do begin
    if Ready and (HLogBuf=0) then
    begin
      HLogBuf := GlobalAlloc(gmem_moveable,InBuffSize);
      LogBuf := nil;
      LogPtr := 0;
      LStart := 0;
      LCount := 0;
      DStart := 0;
      DCount := 0;
    end;
    CreateLogBuf := Ready and (HLogBuf<>0);
  end;
end;

procedure FreeLogBuf;
begin
  with cv do begin
    if (HLogBuf=0) or FileLog or DdeLog then exit;
    if LogBuf<>nil then GlobalUnlock(HLogBUf);
    GlobalFree(HLogBuf);
    HLogBuf := 0;
    LogBuf := nil;
    LogPtr := 0;
    LStart := 0;
    LCount := 0;
    DStart := 0;
    DCount := 0;
  end;
end;

function CreateBinBuf: boolean;
begin
  with cv do begin
    if Ready and (HBinBuf=0) then
    begin
      HBinBuf := GlobalAlloc(gmem_moveable,InBuffSize);
      BinBuf := nil;
      BinPtr := 0;
      BStart := 0;
      BCount := 0;
    end;
    CreateBinBuf := Ready and (HBinBuf<>0);
  end;
end;

procedure FreeBinBuf;
begin
  with cv do begin
    if (HBinBuf=0) or BinLog then exit;
    if BinBuf<>nil then GlobalUnlock(HBinBUf);
    GlobalFree(HBinBuf);
    HBinBuf := 0;
    BinBuf := nil;
    BinPtr := 0;
    BStart := 0;
    BCount := 0;
  end;
end;

procedure FileSendStart;
var
  Option: longint;
begin
  if FSend then exit;
  if ProtoFlag then
  begin
    FreeFileVar(SendVar);
    exit;
  end;

  if not LoadTTFILE then exit;
  if not NewFileVar(SendVar) then
  begin
    FreeTTFILE;
    exit;
  end;
  SendVar^.OpId := OpSendFile;

  FSend := true;

  if (StrLen(@SendVar^.FullName[SendVar^.DirLen])=0) then
  begin
    Option := MakeLong(ts.TransBin,0);
    if not GetTransFname(HMainWin, GTF_Send, @Option, SendVar, ts.FileDir) then
    begin
      FileSendEnd;
      exit;
    end;
    ts.TransBin := LoWord(Option);
  end
  else
    SetFileVar(SendVar);

  with SendVar^ do
  begin
    HelpId := HlpFileSend;
    FileHandle := _lopen(FullName,of_Read);
    FileOpen := FileHandle>0;
    if not FileOpen then
    begin
      FileSendEnd;
      exit;
    end;
    ByteCount := 0;
  end;

  TalkStatus := IdTalkFile;
  FileRetrySend := false;
  FileRetryEcho := false;
  FileCRSend := false;

  SendDlg := PFileTransDlg(Application^.MakeWindow(New(PFileTransDlg,
                 Init(false)) ));
  if SendDlg=nil then FileSendEnd;
end;

procedure FileSendEnd;
begin
  if not FSend then exit;
  TalkStatus := IdTalkKeyb;

  if SendDlg<>nil then
  begin
    SendDlg^.CloseWindow;
    SendDlg := nil;
  end;

  FreeTTFILE;
  FreeFileVar(SendVar);
  FSend := false;
  EndDdeCmnd;
end;

procedure FileSend;

  function Out1(b: byte): integer;
  begin
    if ts.TransBin > 0 then
      Out1 := CommBinaryOut(@cv,@b,1)
    else if (b>=$20) or (b=$09) or (b=$0A) or (b=$0D) then
      Out1 := CommTextOut(@cv,@b,1)
    else
      Out1 := 1;
  end;

  function Echo1(b: byte): integer;
  begin
    if ts.TransBin > 0 then
      Echo1 := CommBinaryEcho(@cv,@b,1)
    else
      Echo1 := CommTextEcho(@cv,@b,1);
  end;

var
  c, fc: word;
begin
  if (SendDlg=nil) or (SendDlg^.Pause) then exit;

  if FileRetrySend then
  begin
    FileRetryEcho := ts.LocalEcho>0;
    c := Out1(FileByte);
    FileRetrySend := c=0;
    if FileRetrySend then exit;
  end;

  if FileRetryEcho then
  begin
    c := Echo1(FileByte);
    FileRetryEcho := c=0;
    if FileRetryEcho then exit;
  end;

  SendDlg^.RefreshNum;

  repeat  
    fc := _lread(SendVar^.FileHandle,@FileByte,1);
    SendVar^.ByteCount := SendVar^.ByteCount + fc;

    if FileCRSend and (fc=1) and (FileByte=$0A) then
    begin
      fc := _lread(SendVar^.FileHandle,@FileByte,1);
      SendVar^.ByteCount := SendVar^.ByteCount + fc;
    end;

    if fc<>0 then
    begin
      c := Out1(FileByte);
      FileCRSend := (ts.TransBin=0) and (FileByte=$0D);
      FileRetrySend := c=0;
      if FileRetrySend then exit;
      if ts.LocalEcho>0 then
      begin
        c := Echo1(FileByte);
        FileRetryEcho := c=0;
        if FileRetryEcho then exit;
      end;
    end;
  until fc=0;

  if fc=0 then FileSendEnd;
end;

function OpenProtoDlg(fv: PFileVar; IdProto, Mode: integer; Opt1, Opt2: WORD): BOOL;
begin
  ProtoDlg := PProtoDlg(Application^.MakeWindow(
           New(PProtoDlg,Init(fv,IdProto,Mode,Opt1,Opt2))));
  OpenProtoDlg := ProtoDlg <> nil;
end;

procedure CloseProtoDlg;
begin
  if ProtoDlg <> nil then
    ProtoDlg^.CloseWindow;
  ProtoDlg := nil;
end;

function ProtoStart: bool;
begin
  ProtoStart := false;
  if ProtoFlag then exit;
  if FSend then
  begin
    FreeFileVar(FileVar);
    exit;
  end;

  if not LoadTTFILE then exit;
  NewFileVar(FileVar);
  cv.FilePause :=
    (FileVar<>nil) or
    (FLogDlg<>nil) and FLogDlg^.Pause;

  if FileVar=nil then
  begin
    FreeTTFILE;
    exit;
  end;
  ProtoFlag := true;
  ProtoStart := true;
end;

procedure ProtoEnd;
begin
  if not ProtoFlag then exit;

  {Enable transmit delay (serial port)}
  cv.DelayFlag := true;
  TalkStatus := IdTalkKeyb;

  CloseProtoDlg;

  FreeTTFILE;
  FreeFileVar(FileVar);
  cv.FilePause :=
    (FileVar<>nil) or
    (FLogDlg<>nil) and FLogDlg^.Pause;

  ProtoFlag := false;
  EndDdeCmnd;
end;

function ProtoDlgParse: integer;
begin
  if (ProtoDlg <> nil) then
    ProtoDlgParse := ProtoDlg^.Parse
  else
    ProtoDlgParse := ActiveWin;
end;

procedure ProtoDlgTimeOut;
begin
  if ProtoDlg <> nil then
    ProtoDlg^.TimeOutProc;
end;

procedure KermitStart(mode: integer);
var
  w: word;
begin
  if not ProtoStart then exit;

  case mode of
    IdKmtSend:
        if (StrLen(@FileVar^.FullName[FileVar^.DirLen])=0) then
        begin
          if not GetMultiFname(HMainWin,FileVar,ts.FileDir,GMF_Kermit,@W) or
            (FileVar^.NumFname=0) then
          begin
            ProtoEnd;
            exit;
          end;
        end
        else begin
          FileVar^.OpId := OpKmtSend;
          SetFileVar(FileVar);
        end;
    IdKmtReceive: FileVar^.OpId := OpKmtRcv;
    IdKmtGet: begin
        if not GetGetFname(HMainWin,FileVar) or (StrLen(FileVar^.FullName)=0) then
        begin
          ProtoEnd;
          exit;
        end;
        FileVar^.OpId := OpKmtGet;
      end;
    IdKmtFinish: FileVar^.OpId := OpKmtFin;
  else
    begin
      ProtoEnd;
      exit;
    end;
  end;
  TalkStatus := IdTalkQuiet;

  {disable transmit delay (serial port)}
  cv.DelayFlag := false;

  if not OpenProtoDlg(FileVar,PROTO_KMT,mode,0,0) then
    ProtoEnd;

end;

procedure XMODEMStart(mode: integer);
var
  Option: longint;
begin
  if not ProtoStart then exit;

  if mode=IdXReceive then
    FileVar^.OpId := OpXRcv
  else
    FileVar^.OpId := OpXSend;

  if (StrLen(@FileVar^.FullName[FileVar^.DirLen])=0) then
  begin
    Option := MakeLong(ts.XmodemBin,ts.XmodemOpt);
    if not GetXFname(HMainWin,mode=IdXReceive,@Option,FileVar,ts.FileDir) then
    begin
      ProtoEnd;
      exit;
    end;
    ts.XmodemOpt := HiWord(Option);
    ts.XmodemBin := LoWord(Option);
  end
  else
    SetFileVar(FileVar);

  if mode=IdXReceive then
    FileVar^.FileHandle := _lcreat(FileVar^.FullName,0)
  else
    FileVar^.FileHandle := _lopen(FileVar^.FullName,of_Read);

  FileVar^.FileOpen := FileVar^.FileHandle>0;
  if not FileVar^.FileOpen then
  begin
    ProtoEnd;
    exit;
  end;
  TalkStatus := IdTalkQuiet;

  {disable transmit delay (serial port)}
  cv.DelayFlag := false;

  if not OpenProtoDlg(FileVar,PROTO_XM,mode,ts.XmodemOpt,ts.XmodemBin) then
    ProtoEnd;

end;

procedure ZMODEMStart(mode: integer);
var
  Opt: word;
begin
  if not ProtoStart then exit;

  if mode=IdZSend then
  begin
    FileVar^.OpId := OpZSend;
    if (StrLen(@FileVar^.FullName[FileVar^.DirLen])=0) then
    begin
      Opt := ts.XmodemBin;
      if (not GetMultiFname(HMainWin,FileVar,ts.FileDir,GMF_Z,@Opt) or
        (FileVar^.NumFname=0)) then
      begin
        ProtoEnd;
        exit;
      end;
      ts.XmodemBin := Opt;
    end
    else
      SetFileVar(FileVar);
  end
  else {IdZReceive or IdZAuto}
    FileVar^.OpId := OpZRcv;

  TalkStatus := IdTalkQuiet;

  {disable transmit delay (serial port)}
  cv.DelayFlag := false;

  if not OpenProtoDlg(FileVar,PROTO_ZM,mode,Opt,0) then
    ProtoEnd;

end;


procedure BPStart(mode: integer);
var
  Option: longint;
begin
  if not ProtoStart then exit;
  if mode=IdBPSend then
  begin
    FileVar^.OpId := OpBPSend;
    if (StrLen(@FileVar^.FullName[FileVar^.DirLen])=0) then
    begin
      if not GetTransFname(HMainWin, GTF_BP, @Option, FileVar, ts.FileDir) then
      begin
        ProtoEnd;
        exit;
      end;
    end
    else
      SetFileVar(FileVar);

  end
  else {IdBPReceive or IdBPAuto}
    FileVar^.OpId := OpBPRcv;

  TalkStatus := IdTalkQuiet;

  {disable transmit delay (serial port)}
  cv.DelayFlag := false;

  if not OpenProtoDlg(FileVar,PROTO_BP,mode,0,0) then
    ProtoEnd;

end;

procedure QVStart(mode: integer);
var
  W: word;
begin
  if not ProtoStart then exit;

  if mode=IdQVSend then
  begin
    FileVar^.OpId := OpQVSend;
    if (StrLen(@FileVar^.FullName[FileVar^.DirLen])=0) then
    begin
      if not GetMultiFname(HMainWin,FileVar,ts.FileDir,GMF_QV, @W) or
         (FileVar^.NumFname=0) then
      begin
        ProtoEnd;
        exit;
      end;
    end
    else
      SetFileVar(FileVar);
  end
  else
    FileVar^.OpId := OpQVRcv;

  TalkStatus := IdTalkQuiet;

  {disable transmit delay (serial port)}
  cv.DelayFlag := false;

  if not OpenProtoDlg(FileVar,PROTO_QV,mode,0,0) then
    ProtoEnd;

end;


{File transfer dialog box}
constructor TFileTransDlg.Init(Log: boolean);
begin
  TDlgWindow.Init(Application^.MainWindow,PChar(Id_FileTransDlg));

  LogFlag := Log;
  Pause := false;
  cv.FilePause :=
    FileVar<>nil;

  if LogFlag then
    FV := LogVar
  else
    FV := SendVar;
end;

destructor TFileTransDlg.Done;
begin
  TDlgWindow.Done;
  if LogFlag then
  begin
    FLogDlg := nil;
    LogEnd;
  end
  else begin
    SendDlg := nil;
    FileSendEnd;
  end;
end;

function TFileTransDlg.GetClassName: PChar;
begin
  GetClassName := 'FileTransDlg';
end;

procedure TFileTransDlg.GetWindowClass(var AWndClass: TWndClass);
begin
  TDlgWindow.GetWindowClass(AWndClass);
  AWndClass.HIcon := LoadIcon(HInstance, PChar(IdTTIcon));
end;

procedure TFileTransDlg.SetupWindow;
begin
  FirstAct := true;
  TDlgWindow.SetupWindow;
  with FV^ do
  begin
    SetWindowText(HWindow, DlgCaption);
    SetDlgItemText(HWindow, id_TransFName, @FullName[DirLen]);
  end
end;

procedure TFileTransDlg.RefreshNum;
var
  NumPStr: string[10];
  NumStr: array[0..12] of char;
begin
  Str(FV^.ByteCount,NumPStr);
  StrPCopy(NumStr,NumPStr);
  SetDlgItemText(HWindow, id_TransBytes, NumStr);
end;

procedure TFileTransDlg.ChangeButton;
begin
  if Pause then
    SetDlgItemText(HWindow, Id_TransPauseStart, '&Start')
  else
    SetDlgItemText(HWindow, Id_TransPauseStart, 'Pau&se');

  cv.FilePause :=
    (FileVar<>nil) or
    LogFlag and Pause;
end;

procedure TFileTransDlg.WMCommand(var Msg: TMessage);
begin
  TDlgWindow.WMCommand(Msg);
  case Msg.wParam of
    Id_TransPauseStart:
      begin
        Pause := not Pause;
        ChangeButton;
      end;
    Id_TransHelp:
      PostMessage(HMainWin,WM_User+MsgDlgHelp2,0,0);
   end;
end;

procedure TFileTransDlg.WMWindowPosChanged(var Msg: TMessage);
begin
  if FirstAct then
  begin
    if LogFlag then
      ShowWindow(HWindow,SW_MINIMIZE);
    FirstAct := false;
  end;
end;

{end of file transfer dialog box}

{Kermit dialog}

constructor TProtoDlg.Init(pfv: PFileVar; ProtoId, Mode: integer; Opt1, Opt2: WORD);
begin
  TDialog.Init(Application^.MainWindow,PChar(Id_ProtoDialog));
  fv := pfv;
  Proto := ProtoId;

  case Proto of
    PROTO_KMT:
      vsize := sizeof(TKmtVar);
    PROTO_XM:
      vsize := sizeof(TXVar);
    PROTO_ZM:
      vsize := sizeof(TZVar);
    PROTO_BP:
      vsize := sizeof(TBPVar);
    PROTO_QV:
      vsize := sizeof(TQVVar);
  end;
  GetMem(pv, vsize);
  if pv <> nil then
    case Proto of
      PROTO_KMT:
        PKmtVar(pv)^.KmtMode := Mode;
      PROTO_XM: begin
          PXVar(pv)^.XMode := Mode;
	  PXVar(pv)^.XOpt := Opt1;
	  PXVar(pv)^.TextFlag := 1 - (Opt2 and 1);
	end;
      PROTO_ZM: begin
	  PZVar(pv)^.BinFlag := (Opt1 and 1) <> 0;
	  PZVar(pv)^.ZMode := Mode;
	end;
      PROTO_BP:
        PBPVar(pv)^.BPMode := Mode;
      PROTO_QV:
        PQVVar(pv)^.QVMode := Mode;
    end;

end;

destructor TProtoDlg.Done;
begin
  KillTimer(HMainWin,IdProtoTimer);
  if (Proto=PROTO_QV) and
     (PQVVar(pv)^.QVMode=IdQVSend) then
    CommTextOut(@cv,#$0D,1);
  if fv^.LogFlag then
    _lclose(fv^.LogFile); 
  fv^.LogFile := 0;
  if pv<>nil then FreeMem(pv,vsize);
  ProtoDlg := nil;
  ProtoEnd;

  TDialog.Done;
end;

procedure TProtoDlg.SetupWindow;
begin
  if pv=nil then
    CloseWindow;
  fv^.HWin := HWindow;

  ProtoInit(Proto,fv,pv,@cv,@ts);

  TDialog.SetupWindow;
end;

procedure TProtoDlg.Cancel(var Msg: TMessage);
begin
  if ProtoCancel(Proto,fv,pv,@cv) then
    TDialog.Cancel(Msg);
end;

function TProtoDlg.Parse: integer;
var
  P: integer;
begin
  P := ActiveWin;
  if ProtoParse(Proto,fv,pv,@cv) then
    P := 0
  else begin
    CommSend(@cv);
    CloseWindow;
  end;
  Parse := P;
end;

procedure TProtoDlg.TimeOutProc;
begin
  ProtoTimeOutProc(Proto,fv,pv,@cv);
end;

{initialization}
begin
 HTTFILE := 0;
 TTFILECount := 0;

 LogVar := nil;
 SendVar :=nil;
 FileVar := nil;
 LogLast := 0;

 FLogDlg := nil;
 SendDlg := nil;

 ProtoDlg := nil;

 FileLog := false;
 BinLog := false;
 DDELog := false;
 FSend := false;
 ProtoFLag := false;
end.
